@@ -37,10 +37,12 @@ using IPod;
 using Hyena;
 using Banshee.Base;
 using Banshee.ServiceStack;
+using Banshee.Sources;
 using Banshee.Dap;
 using Banshee.Hardware;
 using Banshee.Collection.Database;
 using Banshee.Library;
+using Banshee.Playlist;
 
 using Banshee.Dap.Gui;
 
@@ -78,22 +80,48 @@ namespace Banshee.Dap.Ipod
             if (ipod_device == null) {
                 throw new InvalidDeviceException ();
             }
-            
+
             name_path = Path.Combine (Path.GetDirectoryName (ipod_device.TrackDatabasePath), "BansheeIPodName");
-            
+            Name = GetDeviceName ();
+
+            SupportsPlaylists = ipod_device.ModelInfo.DeviceClass != "shuffle";
+
+            // TODO disable this later, but right now it won't disable it in Sync, so might as well
+            // leave it enabled
+            //SupportsPodcasts = ipod_device.ModelInfo.HasCapability ("podcast");
+            SupportsVideo = ipod_device.ModelInfo.DeviceClass == "video" ||
+                            ipod_device.ModelInfo.DeviceClass == "classic" ||
+                            (ipod_device.ModelInfo.DeviceClass == "nano" && ipod_device.ModelInfo.Generation >= 3);
+
             Initialize ();
+
+            AddDapProperty (Catalog.GetString ("Device"), ipod_device.ModelInfo.DeviceClass);
+            AddDapProperty (Catalog.GetString ("Color"), ipod_device.ModelInfo.ShellColor);
+            AddDapProperty (Catalog.GetString ("Generation"), ipod_device.ModelInfo.Generation.ToString ());
+            AddDapProperty (Catalog.GetString ("Capacity"), ipod_device.ModelInfo.AdvertisedCapacity);
+            AddDapProperty (Catalog.GetString ("Serial number"), ipod_device.Serial);
+            AddDapProperty (Catalog.GetString ("Produced on"), ipod_device.ProductionInfo.DisplayDate);
+            AddDapProperty (Catalog.GetString ("Firmware"), ipod_device.FirmwareVersion);
+
+            string [] capabilities = new string [ipod_device.ModelInfo.Capabilities.Count];
+            ipod_device.ModelInfo.Capabilities.CopyTo (capabilities, 0);
+            AddDapProperty (Catalog.GetString ("Capabilities"), String.Join (", ", capabilities));
+            AddYesNoDapProperty (Catalog.GetString ("Supports cover art"), ipod_device.ModelInfo.AlbumArtSupported);
+            AddYesNoDapProperty (Catalog.GetString ("Supports photos"), ipod_device.ModelInfo.PhotosSupported);
         }
 
         public override void Dispose ()
         {
-            ThreadAssist.ProxyToMain (delegate { DestroyUnsupportedView (); });
+            ThreadAssist.ProxyToMain (DestroyUnsupportedView);
             CancelSyncThread ();
             base.Dispose ();
         }
 
         // WARNING: This will be called from a thread!
         protected override void Eject ()
-        {   
+        {
+            base.Eject ();
+            CancelSyncThread ();
             if (ipod_device.CanUnmount) {
                 ipod_device.Unmount ();
             }
@@ -118,6 +146,7 @@ namespace Banshee.Dap.Ipod
         {
             LoadIpod ();
             LoadFromDevice (false);
+            OnTracksAdded ();
         }
 
         private void LoadIpod ()
@@ -135,7 +164,7 @@ namespace Banshee.Dap.Ipod
                     }
                 }
                 database_supported = true;
-                ThreadAssist.ProxyToMain (delegate { DestroyUnsupportedView (); });
+                ThreadAssist.ProxyToMain (DestroyUnsupportedView);
             } catch (DatabaseReadException e) {
                 Log.Exception ("Could not read iPod database", e);
                 ipod_device.LoadTrackDatabase (true);
@@ -151,6 +180,8 @@ namespace Banshee.Dap.Ipod
             }
             
             database_loaded = true;
+
+            Name = GetDeviceName ();
         }
         
         private int CountMusicFiles ()
@@ -178,16 +209,39 @@ namespace Banshee.Dap.Ipod
             }
             
             tracks_map.Clear ();
-             
+
             if (database_supported || (ipod_device.HasTrackDatabase && 
                 ipod_device.ModelInfo.DeviceClass == "shuffle")) {
                 foreach (Track ipod_track in ipod_device.TrackDatabase.Tracks) {
-                    IpodTrackInfo track = new IpodTrackInfo (ipod_track);
-                    track.PrimarySource = this;
-                    track.Save (false);
-                    tracks_map.Add (track.TrackId, track);
+                    try {
+                        IpodTrackInfo track = new IpodTrackInfo (ipod_track);
+                        track.PrimarySource = this;
+                        track.Save (false);
+                        tracks_map.Add (track.TrackId, track);
+                    } catch (Exception e) {
+                        Log.Exception (e);
+                    }
                 }
-            } 
+
+                Hyena.Data.Sqlite.HyenaSqliteCommand insert_cmd = new Hyena.Data.Sqlite.HyenaSqliteCommand (
+                    @"INSERT INTO CorePlaylistEntries (PlaylistID, TrackID)
+                        SELECT ?, TrackID FROM CoreTracks WHERE PrimarySourceID = ? AND ExternalID = ?");
+                foreach (IPod.Playlist playlist in ipod_device.TrackDatabase.Playlists) {
+                    if (playlist.IsOnTheGo) { // || playlist.IsPodcast) {
+                        Console.WriteLine ("have playlist {0} with {1} items but ignoring b/c otg or podcast", playlist.Name, playlist.Tracks.Count);
+                        continue;
+                    }
+                    PlaylistSource pl_src = new PlaylistSource (playlist.Name, this);
+                    pl_src.Save ();
+                    // We use the IPod.Track.Id here b/c we just shoved it into ExternalID above when we loaded
+                    // the tracks, however when we sync, the Track.Id values may/will change.
+                    foreach (IPod.Track track in playlist.Tracks) {
+                        ServiceManager.DbConnection.Execute (insert_cmd, pl_src.DbId, this.DbId, track.Id);
+                    }
+                    pl_src.UpdateCounts ();
+                    AddChildSource (pl_src);
+                }
+            }
             
             /*else {
                 BuildDatabaseUnsupportedWidget ();
@@ -289,50 +343,34 @@ namespace Banshee.Dap.Ipod
                 Log.Exception (e);
             }
             
-            this.name = null;
             ipod_device.Name = name;
             base.Rename (name);
         }
         
-        private string name;
-        public override string Name {
-            get {
-                if (name != null) {
-                    return name;
-                }
-                
-                if (File.Exists (name_path)) {
-                    using (StreamReader reader = new StreamReader (name_path, System.Text.Encoding.Unicode)) {
-                        name = reader.ReadLine ();
-                    }
-                }
-                
-                if (String.IsNullOrEmpty (name) && database_loaded && database_supported) {
-                    name = ipod_device.Name;
-                }
-                    
-                if (!String.IsNullOrEmpty (name)) {
-                    return name;
-                } else if (ipod_device.PropertyExists ("volume.label")) {
-                    name = ipod_device.GetPropertyString ("volume.label");
-                } else if (ipod_device.PropertyExists ("info.product")) {
-                    name = ipod_device.GetPropertyString ("info.product");
-                } else {
-                    name = ((IDevice)ipod_device).Name ?? "iPod";
-                }
-                
-                try {
-                    return name;
-                } finally {
-                    if (!database_loaded) {
-                        name = null;
-                    }
+        private string GetDeviceName ()
+        {
+            string name = null;
+            if (File.Exists (name_path)) {
+                using (StreamReader reader = new StreamReader (name_path, System.Text.Encoding.Unicode)) {
+                    name = reader.ReadLine ();
                 }
             }
-        }
-        
-        public override bool CanActivate {
-            get { return unsupported_view != null; }
+            
+            if (String.IsNullOrEmpty (name) && database_loaded && database_supported) {
+                name = ipod_device.Name;
+            }
+                
+            if (!String.IsNullOrEmpty (name)) {
+                return name;
+            } else if (ipod_device.PropertyExists ("volume.label")) {
+                name = ipod_device.GetPropertyString ("volume.label");
+            } else if (ipod_device.PropertyExists ("info.product")) {
+                name = ipod_device.GetPropertyString ("info.product");
+            } else {
+                name = ((IDevice)ipod_device).Name ?? "iPod";
+            }
+            
+            return name;
         }
         
         public override bool CanRename {
@@ -351,6 +389,22 @@ namespace Banshee.Dap.Ipod
 
 #region Syncing
 
+        protected override void OnTracksAdded ()
+        {
+            if (!IsAdding && tracks_to_add.Count > 0 && !Sync.Syncing) {
+                QueueSync ();
+            }
+            base.OnTracksAdded ();
+        }
+
+        protected override void OnTracksDeleted ()
+        {
+            if (!IsDeleting && tracks_to_remove.Count > 0 && !Sync.Syncing) {
+                QueueSync ();
+            }
+            base.OnTracksDeleted ();
+        }
+        
         private Queue<IpodTrackInfo> tracks_to_add = new Queue<IpodTrackInfo> ();
         private Queue<IpodTrackInfo> tracks_to_remove = new Queue<IpodTrackInfo> ();
         
@@ -360,9 +414,9 @@ namespace Banshee.Dap.Ipod
         private Thread sync_thread;
         private AutoResetEvent sync_thread_wait;
         private bool sync_thread_dispose = false;
-        
+
         public override bool IsReadOnly {
-            get { return ipod_device.IsReadOnly; }
+            get { return ipod_device.IsReadOnly || !database_supported; }
         }
         
         public override void Import ()
@@ -375,18 +429,19 @@ namespace Banshee.Dap.Ipod
             throw new Exception ("Copy to Library is not implemented for iPods yet");
         }*/
 
-        protected override void DeleteTrack (DatabaseTrackInfo track)
+        protected override bool DeleteTrack (DatabaseTrackInfo track)
         {
             lock (sync_mutex) {
                 if (!tracks_map.ContainsKey (track.TrackId)) {
-                    return;
+                    return true;
                 }
                 
                 IpodTrackInfo ipod_track = tracks_map[track.TrackId];
                 if (ipod_track != null) {
                     tracks_to_remove.Enqueue (ipod_track);
-                    QueueSync ();
                 }
+                
+                return true;
             }
         }
         
@@ -403,8 +458,14 @@ namespace Banshee.Dap.Ipod
                 ipod_track.Save (false);
             
                 tracks_to_add.Enqueue (ipod_track);
-                
-                QueueSync ();
+            }
+        }
+
+        public override void SyncPlaylists ()
+        {
+            if (!IsReadOnly && Monitor.TryEnter (sync_mutex)) {
+                PerformSync ();
+                Monitor.Exit (sync_mutex);
             }
         }
 
@@ -415,18 +476,23 @@ namespace Banshee.Dap.Ipod
                     Application.IdleTimeoutRemove (sync_timeout_id);
                 }
                 
-                sync_timeout_id = Application.RunTimeout (1000, PerformSync);
+                sync_timeout_id = Application.RunTimeout (150, PerformSync);
             }
         }
         
         private void CancelSyncThread ()
         {
+            Thread thread = sync_thread;
             lock (sync_mutex) {
                 if (sync_thread != null && sync_thread_wait != null) {
                     sync_thread_dispose = true;
                     sync_thread_wait.Set ();
                 }
             }
+
+            if (thread != null) {
+                thread.Join ();
+            }
         }
         
         private bool PerformSync ()
@@ -453,20 +519,24 @@ namespace Banshee.Dap.Ipod
         
         private void PerformSyncThread ()
         {
-            while (true) {
-                sync_thread_wait.WaitOne ();
-                if (sync_thread_dispose) {
-                    break;
+            try {
+                while (true) {
+                    sync_thread_wait.WaitOne ();
+                    if (sync_thread_dispose) {
+                        break;
+                    }
+                    
+                    PerformSyncThreadCycle ();
                 }
                 
-                PerformSyncThreadCycle ();
-            }
-            
-            lock (sync_mutex) {
-                sync_thread_dispose = false;
-                sync_thread_wait.Close ();
-                sync_thread_wait = null;
-                sync_thread = null;
+                lock (sync_mutex) {
+                    sync_thread_dispose = false;
+                    sync_thread_wait.Close ();
+                    sync_thread_wait = null;
+                    sync_thread = null;
+                }
+            } catch (Exception e) {
+                Log.Exception (e);
             }
         }
         
@@ -480,10 +550,13 @@ namespace Banshee.Dap.Ipod
                 
                 try {
                     track.CommitToIpod (ipod_device);
+                    tracks_map[track.TrackId] = track;
                 } catch (Exception e) {
                     Log.Exception ("Cannot save track to iPod", e);
                 }
             }
+
+            // TODO sync updated metadata to changed tracks
             
             while (tracks_to_remove.Count > 0) {
                 IpodTrackInfo track = null;
@@ -502,13 +575,50 @@ namespace Banshee.Dap.Ipod
                 } catch (Exception e) {
                     Log.Exception ("Cannot remove track from iPod", e);
                 }
-            } 
+            }
+
+            // Remove playlists on the device
+            List<IPod.Playlist> device_playlists = new List<IPod.Playlist> (ipod_device.TrackDatabase.Playlists);
+            foreach (IPod.Playlist playlist in device_playlists) {
+                if (!playlist.IsOnTheGo) { // && !playlist.IsPodcast) {
+                    ipod_device.TrackDatabase.RemovePlaylist (playlist);
+                }
+            }
+            device_playlists.Clear ();
+            
+            if (SupportsPlaylists) {
+                // Add playlists from Banshee to the device
+                foreach (Source child in Children) {
+                    PlaylistSource from = child as PlaylistSource;
+                    if (from != null && from.Count > 0) {
+                        IPod.Playlist playlist = ipod_device.TrackDatabase.CreatePlaylist (from.Name);
+                        foreach (int track_id in ServiceManager.DbConnection.QueryEnumerable<int> (String.Format (
+                            "SELECT CoreTracks.TrackID FROM CoreTracks{0} WHERE {1}",
+                            from.DatabaseTrackModel.JoinFragment, from.DatabaseTrackModel.Condition)))
+                        {
+                            playlist.AddTrack (tracks_map[track_id].IpodTrack);
+                        }
+                    }
+                }
+            }
+    
+            // Sync podcast playlist
+            /*IPod.Playlist podcast_playlist = GetPodcastPlaylist ();
+            podcast_playlist.Clear ();
+            foreach (int track_id in ServiceManager.DbConnection.QueryEnumerable<int> (
+                "SELECT CoreTracks.TrackID FROM CoreTracks WHERE PrimarySourceID = ? AND (Attributes & ?) != 0",
+                DbId, (int)Banshee.Collection.TrackMediaAttributes.Podcast))
+            {
+                podcast_playlist.AddTrack (tracks_map[track_id].IpodTrack);
+            }*/
             
             try {
                 ipod_device.TrackDatabase.SaveStarted += OnIpodDatabaseSaveStarted;
                 ipod_device.TrackDatabase.SaveEnded += OnIpodDatabaseSaveEnded;
                 ipod_device.TrackDatabase.SaveProgressChanged += OnIpodDatabaseSaveProgressChanged;
                 ipod_device.Save ();
+            } catch (InsufficientSpaceException) {
+                ErrorSource.AddMessage (Catalog.GetString ("Out of space on device"), Catalog.GetString ("Please manually remove some songs"));
             } catch (Exception e) {
                 Log.Exception ("Failed to save iPod database", e);
             } finally {
@@ -517,6 +627,19 @@ namespace Banshee.Dap.Ipod
                 ipod_device.TrackDatabase.SaveProgressChanged -= OnIpodDatabaseSaveProgressChanged;
             }
         }
+
+        /*private IPod.Playlist GetPodcastPlaylist ()
+        {
+            foreach (IPod.Playlist playlist in ipod_device.TrackDatabase.Playlists) {
+                if (playlist.IsPodcast) {
+                    return playlist;
+                }
+            }
+
+            IPod.Playlist podcast_playlist= ipod_device.TrackDatabase.CreatePlaylist (Catalog.GetString ("Podcasts"));
+            podcast_playlist.IsPodcast = true;
+            return podcast_playlist;
+        }*/
         
         private UserJob sync_user_job;
         
